"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const glob_1 = __importDefault(require("glob"));
const index_1 = require("../../index");
const utils_1 = require("../../utils/utils");
class BaseGenerator extends index_1.Generator {
    constructor(options, config) {
        super(options);
        this.needsProject = true;
        this.type = config.type;
        this.ext = config.ext.toLowerCase();
        this.allowExtends = !!config.allowExtends;
        let allowExtendsWithoutBase = config.allowExtendsWithoutBase;
        if (allowExtendsWithoutBase === false) {
            this.allowExtendsWithoutBase = false;
        }
        else {
            this.allowExtendsWithoutBase = true;
        }
        this.fileExtension = !config.noFileExtension;
        this.declaration = !!config.declaration;
        let type = this.type.toLowerCase();
        this.srcRoot(type);
        this.destRoot(`app/src/${utils_1.pluralize(type)}`);
    }
    defineOptions() {
        this.option('name', {
            aliases: ['n'],
            description: `The name of the ${this.type}`,
        });
        this.option('dir', {
            aliases: ['d'],
            description: `Specify the path relative to \`app/src/${utils_1.pluralize(this.type.toLowerCase())}\` in which to store the ${this.type}`,
            defaults: '',
        });
        if (this.allowExtends || this.allowExtendsWithoutBase) {
            this.option('extends', {
                aliases: ['x'],
                description: `Specify the relative import path to the ${this.type} to extend`,
            });
        }
        if (this.declaration) {
            this.option('declaration', {
                aliases: ['dts'],
                description: 'Do not create a declaration file.',
                defaults: true,
            });
        }
        this.option('register', {
            aliases: ['reg'],
            description: `Do not register the ${this.type} with the framework`,
            defaults: true,
        });
    }
    askQuestions() {
        let options = this.options, name = options.name, promise;
        if (!this.utils.isString(options.name) && this.commands.length > 1) {
            name = this.commands[1];
            promise = this.ui
                .prompt([
                {
                    name: 'name',
                    type: this.ui.PROMPTS.CONFIRM,
                    default: true,
                    message: `Did you mean \`${this.buildFullCommand()
                        .slice(0, -1)
                        .join(' ')} -n ${name}\`?`,
                },
            ])
                .then((answer) => {
                if (answer.name) {
                    return name;
                }
                return this.promptName(options.name);
            });
        }
        else {
            promise = this.promptName(options.name);
        }
        return promise.then((name) => {
            options.name = name;
        });
    }
    run() {
        let options = this.options, name = options.name, config = {
            name: name,
            type: name.replace(/[A-Z]/g, (match, index) => {
                if (index > 0) {
                    return '-' + match;
                }
                return match;
            }),
            register: options.register !== false,
        }, dest = this.getDestination();
        let ext = options.extends, src = this.ext;
        if (ext === false && this.allowExtends) {
            src = `base.${src}`;
        }
        else {
            config.ext = this.findExtends(dest)
                .replace(/\\/g, '/')
                .replace(/'/g, '');
        }
        if (this.allowExtends) {
            let root = this.destRoot();
            config.type = path_1.default
                .relative(path_1.default.resolve(root), path_1.default.resolve(root, dest, '..'))
                .replace(/\\|\//g, '-');
        }
        return this._render(src, dest, config).then(() => {
            return Promise.all([this.processMainTs(), this.processMainLess()]);
        });
    }
    promptName(name) {
        if (this.utils.isString(name)) {
            return Promise.resolve(name);
        }
        return this.ui
            .prompt([
            {
                name: 'name',
                type: 'input',
                message: `What is the name of your ${this.type}?`,
            },
        ])
            .then((answer) => {
            return answer.name;
        });
    }
    _render(src, dest, config) {
        let promises = [this.render(`${src}.ts`, `${dest}.ts`, config)];
        if (this.declaration && this.options.declaration) {
            let dDest = dest.substring(0, dest.lastIndexOf('/')) +
                '/i' +
                dest.substring(dest.lastIndexOf('/') + 1);
            promises.push(this.render(`i${src}.d.ts`, `${dDest}.d.ts`, config));
        }
        return Promise.all(promises);
    }
    processMainTs() {
        let destRoot = path_1.default.resolve(this.destRoot(), '..'), root = path_1.default.relative(this.project.root, destRoot), paths = [
            root + '/app/app.ts',
            root + '/injectables/**/*.ts',
            root + '/attributecontrols/**/*.ts',
            root + '/templatecontrols/**/*.ts',
        ], ignore = [
            root + '/injectables/**/*.d.ts',
            root + '/attributecontrols/**/*.d.ts',
            root + '/templatecontrols/**/*.d.ts',
        ];
        let file = path_1.default.resolve(destRoot, 'main.ts');
        return Promise.all([
            this.file.read(file),
            this.glob(root, paths, ignore),
        ])
            .then((results) => {
            let data = results[0], files = results[1], req = `import '`, append = [];
            files.forEach((file) => {
                if (data.indexOf(file) === -1) {
                    append.push(`${req}${file}';`);
                }
            });
            if (append.length > 0) {
                append.push('');
            }
            return this.formatMain(data, append);
        })
            .then((data) => {
            return this.file.write(file, data);
        });
    }
    processMainLess() {
        let destRoot = path_1.default.resolve(this.destRoot(), '..'), root = path_1.default.relative(this.project.root, destRoot), paths = [
            root + '/attributecontrols/**/*.less',
            root + '/templatecontrols/**/*.less',
            root + '/viewcontrols/**/*.less',
        ];
        let file = path_1.default.resolve(destRoot, '../styles/main.less');
        return Promise.all([
            this.file.read(file),
            this.glob(root, paths, undefined, '../src/'),
        ])
            .then((results) => {
            let data = results[0], files = results[1], imprt = `@import '`, append = [];
            files.forEach((file) => {
                if (data.indexOf(file) === -1) {
                    append.push(`${imprt}${file}';`);
                }
            });
            if (append.length > 0) {
                append.push('');
            }
            return this.formatMain(data, append);
        })
            .then((data) => {
            return this.file.write(file, data);
        });
    }
    glob(root, files, ignore = [], replace = './') {
        if (!this.utils.isArray(ignore)) {
            ignore = [];
        }
        files = files.slice(0);
        let firstFile = files.shift();
        return new Promise((resolve, reject) => {
            glob_1.default(firstFile, {
                ignore: ignore.map((file) => {
                    if (file[0] === '!') {
                        return file.slice(1);
                    }
                    return file;
                }),
            }, (err, matches) => {
                let out = matches.map((file) => {
                    return (replace +
                        path_1.default
                            .relative(root, file)
                            .replace(/\\/g, '/')
                            .replace(/\.ts$/, ''));
                });
                resolve(out);
            });
        }).then((out) => {
            if (files.length > 0) {
                return this.glob(root, files, ignore, replace).then((result) => {
                    return out.concat(result);
                });
            }
            return out;
        });
    }
    getDestination() {
        let options = this.options, name = options.name, dir = options.dir, dest;
        if (this.utils.isEmpty(dir)) {
            dest = name;
        }
        else {
            if (dir[dir.length - 1] === '/') {
                dir = dir.slice(0, -1);
            }
            dest = dir + '/' + name;
        }
        dest += `/${name}`;
        if (this.fileExtension) {
            dest += `.${this.ext}`;
        }
        return dest.toLowerCase();
    }
    findExtends(root) {
        let ext = this.options.extends;
        if (this.utils.isString(ext)) {
            return ext;
        }
        if (!this.allowExtends && this.allowExtendsWithoutBase) {
            return '';
        }
        let dest = this.destRoot();
        return path_1.default.relative(path_1.default.resolve(dest, root, '..'), path_1.default.resolve(dest, `base/base.${this.ext}`));
    }
    formatMain(data, append) {
        let eol = this.file.eol(data), appFound = false, acFound = false, injFound = false, tcFound = false, vcFound = false;
        let sorted = this.sortMain(data.split(eol).concat(append))
            .map((value, index, lines) => {
            if (value.indexOf('/app/') > -1 && !appFound) {
                appFound = true;
                value = eol + value;
            }
            else if (value.indexOf('/attributecontrols') > -1 &&
                !acFound) {
                acFound = true;
                value = eol + value;
            }
            else if (value.indexOf('/injectables') > -1 && !injFound) {
                injFound = true;
                value = eol + value;
            }
            else if (value.indexOf('/templatecontrols') > -1 &&
                !tcFound) {
                tcFound = true;
                value = eol + value;
            }
            else if (value.indexOf('/viewcontrols') > -1 && !vcFound) {
                vcFound = true;
                value = eol + value;
            }
            return value;
        })
            .join(eol)
            .split(eol)
            .filter((value, index, array) => {
            let empty = this.utils.isEmpty(value.trim()), out = !empty || !this.utils.isEmpty(array[index - 1]);
            return out;
        });
        if (sorted[sorted.length - 2].trim() === '' &&
            sorted[sorted.length - 1].trim() === '') {
            sorted.pop();
        }
        if (sorted[sorted.length - 1].trim() !== '') {
            sorted.push('');
        }
        return sorted.join(eol);
    }
    sortMain(lines) {
        let imports = [], others = [], trim, isString = this.utils.isString, isEmpty = this.utils.isEmpty;
        lines.forEach((line) => {
            trim = line.trim();
            if ((trim.indexOf('require') === 0 ||
                trim.indexOf('import') === 0 ||
                trim.indexOf('@import') === 0) &&
                this.hasComponent(trim)) {
                imports.push(line);
            }
            else {
                let last = others[others.length - 1];
                if (isEmpty(trim) && isString(last) && isEmpty(last.trim())) {
                    return;
                }
                others.push(line);
            }
        });
        imports = imports.sort();
        return others.concat(imports);
    }
    hasComponent(line) {
        return (line.indexOf('app/app') > -1 ||
            line.indexOf('attributecontrols') > -1 ||
            line.indexOf('injectables') > -1 ||
            line.indexOf('templatecontrols') > -1 ||
            line.indexOf('viewcontrols') > -1);
    }
    sort(a, b) {
        let aa = a.indexOf('app') > -1, ba = b.indexOf('app') > -1, aac = a.indexOf('attributecontrols') > -1, bac = b.indexOf('attributecontrols') > -1, ai = a.indexOf('injectables') > -1, bi = b.indexOf('injectables') > -1, atc = a.indexOf('templatecontrols') > -1, btc = b.indexOf('templatecontrols') > -1, avc = a.indexOf('viewcontrols') > -1, bvc = b.indexOf('viewcontrols') > -1;
        if (!(aa || aac || ai || atc || avc)) {
            return -1;
        }
        else if (!(ba || bac || bi || btc || bvc)) {
            return 1;
        }
        else if ((aa && ba) ||
            (aac && bac) ||
            (ai && bi) ||
            (atc && btc) ||
            (avc && bvc)) {
            if (a > b) {
                return 1;
            }
            else if (b > a) {
                return -1;
            }
            return 0;
        }
        else if ((ai && btc) ||
            (aac && (bi || btc)) ||
            (aa && (bac || bi || btc))) {
            return -1;
        }
        return 1;
    }
}
exports.default = BaseGenerator;
