"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const base_1 = __importDefault(require("./base"));
const environment_1 = __importDefault(require("../environment/environment"));
const validation_1 = __importDefault(require("../errors/validation"));
const notimplemented_1 = __importDefault(require("../errors/notimplemented"));
const minimist_1 = __importDefault(require("minimist"));
const fileutils_1 = __importDefault(require("./fileutils"));
const processutils_1 = __importDefault(require("./processutils"));
const utils_1 = require("../utils/utils");
class Command extends base_1.default {
    constructor(options) {
        super(options);
        this.needsProject = false;
        this.options = {};
        this._options = {};
        this.file = this.instantiate(fileutils_1.default, options);
        this.process = this.instantiate(processutils_1.default, options);
        this.env = new environment_1.default(options);
        this.parent = options.parent;
        this.option('help', {
            aliases: ['h'],
            description: 'Get help information for this command',
        });
        this.option('verbose', {
            description: 'Print all log statements',
        });
        this.option('silent', {
            description: 'Print only warnings and errors',
        });
        this.option('loglevel', {
            description: 'Print at and above a particular level                (ERROR > WARN > INFO > DEBUG > TRACE)',
        });
    }
    help(command) {
        if (!this.utils.isEmpty(command)) {
            this.ui.help(`Help for command \`${this.buildFullCommand().join(' ')}\`:` +
                os_1.EOL);
        }
        return Promise.resolve(this.generalHelp(command))
            .then(() => {
            return this.commandsHelp(command);
        })
            .then(() => {
            return this.aliasesHelp(command);
        })
            .then(() => {
            return this.optionsHelp(command);
        })
            .then(() => {
            this.ui.help('');
        });
    }
    validateAndRun(commandArgs) {
        if (this.needsProject && !this.utils.isString(this.project.root)) {
            return Promise.reject(new validation_1.default(`This command can only be run inside a project.`));
        }
        this.args = commandArgs.slice(0);
        this._originalArgs = this.args.slice(0);
        this.defineOptions();
        this.parseOptions();
        let commands = (this.commands = this.options._);
        if (this.needsHelp()) {
            return Promise.resolve(this.help(commands[0]));
        }
        return Promise.resolve(this.askQuestions())
            .then(this.validate.bind(this))
            .then((valid) => {
            let command = commands.shift();
            this.args.splice(this.args.indexOf(command), 1);
            if (valid === false) {
                let message;
                if (this.utils.isString(command)) {
                    message = `\`${command}\` doesn't have enough information to complete. For more information see \`platypi ${command} -h\``;
                }
                else {
                    message =
                        'Please specify a command. Use `platypi -h` for more information.';
                }
                throw new validation_1.default(message);
            }
            return this.run();
        });
    }
    defineOptions() { }
    askQuestions() { }
    validate() {
        return true;
    }
    run() {
        throw new notimplemented_1.default(`The command \`${this.constructor.commandName}\` exists, but has not been implemented.`);
    }
    buildFullCommand() {
        let parent = this;
        while (this.utils.isObject(parent.parent)) {
            parent = parent.parent;
        }
        return [this.project.bin].concat(minimist_1.default(parent._originalArgs)._);
    }
    generalHelp(command) {
        this.ui.help(`
  General Usage:

    ${this.buildFullCommand().join(' ')} [...options]`);
    }
    commandsHelp(command) { }
    aliasesHelp(command) {
        let aliases = this.constructor.aliases;
        if (this.utils.isArray(aliases)) {
            aliases = aliases.slice(0);
            let commandName = this.constructor.commandName;
            if (this.utils.isString(commandName)) {
                aliases.unshift(commandName);
            }
            this.utils.remove(aliases, (alias) => alias === command);
            if (aliases.length > 0) {
                this.ui.help(`
  Aliases:
    ${aliases.join(', ')}`);
            }
        }
    }
    optionsHelp(command) {
        this.ui.help(`
  Options:`);
        let options = this._options, longest = 0, lines = [];
        this.utils.forEach(options, (option) => {
            let prepend = '--', aliasPrepend = '-';
            if (option.defaults === true) {
                prepend += 'no-';
                aliasPrepend += '-no-';
            }
            let commands;
            if (this.utils.isArray(option.aliases) &&
                option.aliases.length > 0) {
                commands = `${option.aliases
                    .map((alias) => `${aliasPrepend}${alias}`)
                    .join(', ')}, ${prepend}${option.name}`;
            }
            else {
                commands = `${prepend}${option.name}`;
            }
            if (commands.length > longest) {
                longest = commands.length;
            }
            lines.push({
                command: commands,
                description: option.description,
                defaults: option.defaults,
            });
        });
        let utils = this.utils, isNull = (val) => {
            return utils.isNull(val) || utils.isUndefined(val);
        }, isString = utils.isString, isEmpty = utils.isEmpty, isBoolean = utils.isBoolean;
        this.utils.forEach(lines, (line) => {
            let padding = os_1.EOL + this.file.spaces(longest + 10);
            this.ui.help(`    ${line.command}${this.utils
                .fill(Array(longest - line.command.length + 4), ' ')
                .join('')}${utils_1.wrap(line.description, 58, padding)}`);
            if (!isNull(line.defaults) &&
                !(isString(line.defaults) && isEmpty(line.defaults)) &&
                !isBoolean(line.defaults)) {
                this.ui.help(`      default: ${line.defaults}
`);
            }
        });
    }
    parseOptions() {
        let options = this._options;
        this.options = minimist_1.default(this.args, {
            '--': true,
            default: this.utils.reduce(options, (prev, current) => {
                prev[current.name] = current.defaults;
                return prev;
            }, {}),
            alias: this.utils.reduce(options, (prev, current) => {
                prev[current.name] = current.aliases;
                return prev;
            }, {}),
        });
    }
    option(name, config) {
        if (!this.utils.isObject(config)) {
            config = {};
        }
        this.utils.defaults(config, {
            name: name,
            aliases: [],
            description: 'Description for ' + name,
            defaults: undefined,
            hide: false,
        });
        let _options = this._options, options = this.options;
        if (!this.utils.isObject(_options[name])) {
            _options[name] = config;
        }
        if (!this.utils.isObject(options[name])) {
            options[name] = config.defaults;
        }
        return this;
    }
    needsHelp() {
        return this.options.help && this.commands.length <= 1;
    }
}
exports.default = Command;
Command.commandName = 'command';
Command.aliases = [];
