"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_1 = __importDefault(require("path"));
const base_1 = __importDefault(require("./base"));
class FileUtils extends base_1.default {
    read(source, options = {}) {
        source = path_1.default.normalize(source);
        this.ui.debug(`Reading from \`${source}\``);
        this.utils.defaults(options, {
            encoding: 'utf8',
        });
        return new Promise((resolve, reject) => {
            fs_extra_1.default.readFile(source, options, (err, data) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(data);
            });
        });
    }
    write(dest, data, options = {}) {
        dest = path_1.default.normalize(dest);
        this.ui.debug(`Writing to \`${dest}\``);
        this.utils.defaults(options, {
            encoding: 'utf8',
        });
        return this.ensureWritable(dest).then(() => {
            return new Promise((resolve, reject) => {
                fs_extra_1.default.writeFile(dest, data, options, (err) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve();
                });
            });
        });
    }
    copy(src, dest) {
        return new Promise((resolve, reject) => {
            fs_extra_1.default.copy(src, dest, (err) => {
                if (this.utils.isObject(err)) {
                    return reject(err);
                }
                resolve();
            });
        });
    }
    mkdir(...dirs) {
        return Promise.all(dirs.map((dir) => {
            return mkdirp_1.default(dir);
        })).then(this.utils.noop);
    }
    eol(data) {
        let cr = '\r', lf = '\n', r = /\r/.test(data), n = /\n/.test(data);
        if (r && n) {
            return cr + lf;
        }
        return lf;
    }
    spaces(length) {
        return this.utils.fill(Array(length), ' ').join('');
    }
    mapLines(handler, data) {
        let eol = this.eol(data), lines = data.split(eol);
        return this.utils.map(lines, handler).join(eol);
    }
    dir(src, ignores = []) {
        return new Promise((resolve, reject) => {
            fs_extra_1.default.readdir(path_1.default.resolve(src), (err, directories) => {
                if (this.utils.isObject(err)) {
                    return reject(err);
                }
                let ignoreRegex = ignores.filter((ignore) => {
                    return this.utils.isRegExp(ignore);
                });
                directories = directories.filter((dir) => {
                    let isDir = fs_extra_1.default.statSync(path_1.default.join(src, dir)).isDirectory(), index = ignores.indexOf(dir);
                    if (!isDir || index > -1) {
                        return false;
                    }
                    return !ignoreRegex.some((regex) => {
                        return regex.test(dir);
                    });
                });
                resolve(directories);
            });
        });
    }
    requireAll(src, directories) {
        let modules = {};
        directories.forEach((dir) => {
            let module = require(path_1.default.resolve(src, dir));
            if (this.utils.isObject(module) &&
                this.utils.isObject(module.default)) {
                modules[dir] = module.default;
            }
            else {
                modules[dir] = module;
            }
        });
        return modules;
    }
    ensureWritable(file) {
        return this.mkdir(path_1.default.dirname(file));
    }
}
exports.default = FileUtils;
