"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const chalk_1 = __importDefault(require("chalk"));
const through_1 = __importDefault(require("through"));
const inquirer_1 = __importDefault(require("inquirer"));
const os_1 = require("os");
let Progress = require('pleasant-progress');
let LOG_LEVEL = {
    ERROR: 5,
    WARN: 4,
    INFO: 3,
    DEBUG: 2,
    TRACE: 1,
};
let PROMPTS = {
    INPUT: 'input',
    EXPAND: 'expand',
    CONFIRM: 'confirm',
    LIST: 'list',
    RAWLIST: 'rawlist',
    PASSWORD: 'password',
};
class Ui {
    constructor(options) {
        this.options = options;
        this.LOG_LEVEL = Ui.LOG_LEVEL;
        this.PROMPTS = Ui.PROMPTS;
        this.Prompt = inquirer_1.default.ui.Prompt;
        this.Promise = Promise;
        this.through = through_1.default;
        this.inquirer = inquirer_1.default;
        this.utils = lodash_1.default;
        let progress = (this.progress = new Progress());
        this.output = this.through(function (data) {
            progress.stop(true);
            this.emit('data', data);
        });
        this.output.setMaxListeners(0);
        this.output.pipe(options.output);
        this.input = options.input;
        this.setLogLevel(options.logLevel);
    }
    error(error) {
        if (!error) {
            return;
        }
        let message = error.message, stack = error.stack;
        if (this.utils.isString(stack)) {
            this.logLine(this.makePretty(stack.slice(0, stack.indexOf(message) + message.length), chalk_1.default.red), LOG_LEVEL.ERROR);
            this.logLine(stack.slice(stack.indexOf(message) + message.length + 1), LOG_LEVEL.ERROR);
            return;
        }
        if (this.utils.isString(message)) {
            this.logLine(this.makePretty(message, chalk_1.default.red), LOG_LEVEL.ERROR);
        }
        else {
            this.logLine(this.makePretty(error, chalk_1.default.red), LOG_LEVEL.ERROR);
        }
    }
    warn(message) {
        this.logLine(this.makePretty(message, chalk_1.default.yellow), LOG_LEVEL.WARN);
    }
    info(message) {
        this.logLine(message, LOG_LEVEL.INFO);
    }
    debug(message) {
        this.logLine(this.makePretty(message, chalk_1.default.green), LOG_LEVEL.DEBUG);
    }
    trace(message) {
        this.logLine(this.makePretty(message, chalk_1.default.magenta), LOG_LEVEL.TRACE);
    }
    help(message) {
        this.logLine(message, 999);
    }
    log(message, logLevel = LOG_LEVEL.INFO) {
        if (this.utils.isString(message) && message.indexOf('\u001b') === -1) {
            message = message.replace(/`[^`]*`/g, (substr) => {
                return chalk_1.default.cyan(substr);
            });
        }
        if (this.shouldLog(logLevel)) {
            this.output.write(message);
        }
    }
    logLine(message, logLevel) {
        this.log(message + os_1.EOL, logLevel);
    }
    prompt(questions) {
        questions = questions.map((question) => {
            if (this.utils.isString(question.message)) {
                question.message = this.makePretty(question.message, chalk_1.default.reset);
            }
            return question;
        });
        return this.Promise.resolve(this.inquirer.prompt(questions));
    }
    startProgress(message, stepString) {
        if (!this.shouldLog(LOG_LEVEL.INFO)) {
            return;
        }
        this.progress.start(message, stepString);
    }
    stopProgress(printWithFullStepString) {
        if (this.shouldLog(LOG_LEVEL.INFO)) {
            this.progress.stop(printWithFullStepString);
        }
    }
    setLogLevel(level) {
        if (this.utils.isNumber(LOG_LEVEL[level])) {
            this.logLevel = LOG_LEVEL[level];
            return;
        }
        else if (level < LOG_LEVEL.TRACE) {
            this.logLevel = LOG_LEVEL.TRACE;
            return;
        }
        else if (level > LOG_LEVEL.ERROR) {
            this.logLevel = LOG_LEVEL.ERROR;
            return;
        }
        this.logLevel = level;
    }
    makePretty(message, color) {
        if (!this.utils.isString(message)) {
            return color(message);
        }
        return this.utils
            .map(message.split(/(`[^`]*`)/g), (val) => {
            if (val[0] === '`') {
                return chalk_1.default.cyan(val);
            }
            return color(val);
        })
            .join('');
    }
    shouldLog(logLevel) {
        return logLevel >= (this.logLevel || LOG_LEVEL.INFO);
    }
}
exports.default = Ui;
Ui.LOG_LEVEL = LOG_LEVEL;
Ui.PROMPTS = PROMPTS;
