"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const ui_1 = __importDefault(require("../../../src/ui/ui"));
const through_1 = __importDefault(require("through"));
chai_1.use(require('chai-as-promised'));
chai_1.use(require('sinon-chai'));
describe('Ui', () => {
    let ui, spy;
    beforeEach(() => {
        ui = new ui_1.default({
            input: through_1.default((data) => { }),
            output: through_1.default((data) => { }),
        });
        spy = sinon_1.spy(ui, 'log');
    });
    afterEach(() => {
        spy.restore();
    });
    it('should log at info by default', () => {
        let shouldLogSpy = sinon_1.spy(ui, 'shouldLog');
        ui.log('test');
        chai_1.expect(spy).to.have.been.calledOnce;
        chai_1.expect(spy.lastCall.args[1]).to.be.an('undefined');
        chai_1.expect(shouldLogSpy.lastCall.args[0]).to.equal(ui_1.default.LOG_LEVEL.INFO);
        shouldLogSpy.restore();
    });
    it('should prompt asynchronously', (done) => {
        let promptStub = sinon_1.stub(ui.inquirer, 'prompt').callsFake(() => { });
        chai_1.expect(ui.prompt([]))
            .to.eventually.be.an('undefined')
            .notify(done);
        promptStub.restore();
    });
    describe('progress', () => {
        ['start', 'stop'].forEach((value) => {
            let progressStub;
            beforeEach(() => {
                progressStub = sinon_1.stub(ui.progress, value);
            });
            afterEach(() => {
                progressStub.restore();
            });
            it(`should ${value} at info level`, () => {
                ui[value + 'Progress']();
                chai_1.expect(progressStub).to.have.been.calledOnce;
            });
            it(`should ${value} below info level`, () => {
                ui.setLogLevel(ui_1.default.LOG_LEVEL.TRACE);
                ui[value + 'Progress']();
                chai_1.expect(progressStub).to.have.been.calledOnce;
            });
            it(`should not ${value} above info level`, () => {
                ui.setLogLevel(ui_1.default.LOG_LEVEL.WARN);
                ui[value + 'Progress']();
                chai_1.expect(progressStub).not.to.have.been.called;
            });
        });
    });
    function errorMethod() {
        it('should do nothing if called with falsy values', () => {
            let err;
            ui.error(err);
            err = null;
            ui.error(err);
            err = 0;
            ui.error(err);
            err = '';
            ui.error(err);
            chai_1.expect(spy).not.to.have.been.called;
        });
        it('should accept a string', () => {
            let err = 'test message';
            ui.error(err);
            chai_1.expect(spy).to.have.been.calledOnce;
            chai_1.expect(spy.lastCall.args[1]).to.equal(ui_1.default.LOG_LEVEL.ERROR);
        });
        it('should accept an object', () => {
            let err = {
                message: 'test message',
            };
            ui.error(err);
            chai_1.expect(spy).to.have.been.calledOnce;
            chai_1.expect(spy.lastCall.args[1]).to.equal(ui_1.default.LOG_LEVEL.ERROR);
        });
        it('should accept an error', () => {
            let err = new Error('test message');
            ui.error(err);
            chai_1.expect(spy).to.have.been.calledTwice;
            chai_1.expect(spy.lastCall.args[1]).to.equal(ui_1.default.LOG_LEVEL.ERROR);
        });
    }
    ['error', 'warn', 'info', 'debug', 'trace'].forEach((value) => {
        let level = ui_1.default.LOG_LEVEL[value.toUpperCase()];
        describe(`${value} method`, () => {
            beforeEach(() => {
                ui.setLogLevel(value.toUpperCase());
            });
            if (value === 'error') {
                errorMethod();
            }
            it(`should log at a ${value} level`, () => {
                ui[value]('message');
                chai_1.expect(spy).to.have.been.calledOnce;
                chai_1.expect(spy.lastCall.args[1]).to.equal(level);
            });
            it(`should log below ${value} level`, () => {
                ui.setLogLevel(level - 1);
                ui[value]('message');
                chai_1.expect(spy).to.have.been.calledOnce;
                chai_1.expect(spy.lastCall.args[1]).to.equal(level);
            });
            it(`should not log above ${value} level`, () => {
                let outputSpy = sinon_1.spy(ui.output, 'write');
                ui.setLogLevel(level + 1);
                ui[value]('message');
                if (value !== 'error') {
                    chai_1.expect(outputSpy).not.to.have.been.called;
                }
                else {
                    chai_1.expect(outputSpy).to.have.been.called;
                    chai_1.expect(spy.lastCall.args[1]).to.equal(level);
                }
                outputSpy.restore();
            });
        });
    });
});
